@@ -1,15 +1,194 @@
 import datetime
 from graph_op import CHGraph
 from clickhouse_driver import Client
+from db_op import DBoperator
 import json
 
 
 def is_json(json_string):
-  try:
-    json_object = json.loads(json_string)
-  except ValueError:
-    return False
-  return True
+    try:
+        json_object = json.loads(json_string)
+    except ValueError:
+        return False
+    return True
+
+
+def model_service_ga_build(data, config_params):
+    if "graph" in data.keys():
+        graphName = data["graph"]
+    else:
+        graphName = None
+
+    if "sql" in data.keys():
+        sql = data["sql"]
+    else:
+        sql = None
+
+    if "type" in data.keys():
+        type = data["type"]
+    else:
+        type = None
+
+    if "attrType" in data.keys():
+        attrType = data["attrType"]
+    else:
+        attrType = None
+
+    if "subGraph" in data.keys():
+        subGraph = data["subGraph"]
+    else:
+        subGraph = None
+
+    graph = config_params["graph"]
+    db = config_params["db"]
+    graph.use_graph(graphName, db)
+
+    graph_cfg = db.use_tables(graphName)
+
+    db.insert_tables(subGraph, json.dumps(graph_cfg), "sub", graphName)
+    graph.create_subgraph(subGraph)
+
+    dict = graph_cfg[type][attrType]
+    graph.use_graph(subGraph, db)
+    graph.update_subgraph_by_sql(subGraph, dict, sql, type, attrType, graphName)\
+
+    graphNodes = []
+    for vertex in graph_cfg["vertexes"]:
+        graphNode = {}
+        dict1 = graph_cfg["vertexes"][vertex]
+        tem = []
+        tem.append(dict1["id"])
+        tem.append(dict1["label"])
+        for field in dict1["fields"]:
+            tem.append(field)
+        try:
+            vertex_query = graph.query_vertexes(
+                vertex,
+                [""],
+                tem,
+                "df"
+            )
+        except Exception as e:
+            print(e)
+            return "vertex query failed"
+        graphNode["type"] = vertex
+        graphNode["data"] = vertex_query
+
+        graphNodes.append(graphNode)
+
+    graphEdges = []
+    for edge in graph_cfg["edges"]:
+        graphEdge = {}
+        dict2 = graph_cfg["edges"][edge]
+        tem = []
+        tem.append(dict2["src"])
+        tem.append(dict2["dst"])
+        tem.append(dict2["rank"])
+        for field in dict2["fields"]:
+            tem.append(field)
+        try:
+            edges_result = graph.query_edges(
+                edge,
+                [""],
+                tem,
+                "df"
+            )
+        except Exception as e:
+            print(e)
+            return "edge query failed"
+        graphEdge["type"] = edge
+        graphEdge["data"] = edges_result
+        graphEdge["id"] = edge
+
+        graphEdges.append(graphEdge)
+
+    GraphSet = {}
+    GraphSet["pathList"] = {}
+    GraphSet["columns"] = []
+    GraphSet["rowList"] = []
+    GraphSet["pathList"]["graphEdges"] = graphEdges
+    GraphSet["pathList"]["graphNodes"] = graphNodes
+    return GraphSet
+
+
+def model_service_graph(data, config_params):
+    if "is_source" in data.keys():
+        is_source = data["is_source"]
+    else:
+        is_source = None
+
+    if "source_graph_name" in data.keys():
+        source_graph_name = data["source_graph_name"]
+    else:
+        source_graph_name = None
+
+    if "graph_name" in data.keys():
+        graph_name = data["graph_name"]
+    else:
+        graph_name = None
+
+    db = config_params["db"]
+    res = db.show_tables(is_source, source_graph_name, graph_name)
+    return res
+
+
+def model_service_graph_insert(data, config_params):
+    if "graph_name" in data.keys():
+        graph_name = data["graph_name"]
+    else:
+        return "graph_name shouldn't be null"
+
+    if "graph_cfg" in data.keys():
+        graph_cfg = data["graph_cfg"]
+        if not is_json(graph_cfg):
+            return "schema is not json string"
+    if "is_source" in data.keys():
+        is_source = data["is_source"]
+    else:
+        return "is_source shouldn't be null"
+
+    if "source_graph_name" in data.keys():
+        source_graph_name = data["source_graph_name"]
+    else:
+        return "source_graph_name shouldn't be null"
+
+    db = config_params["db"]
+    res = db.insert_tables(graph_name, graph_cfg, is_source, source_graph_name)
+
+    graph = config_params["graph"]
+    graph.use_graph(source_graph_name, db)
+    graph.create_subgraph(graph_name)
+
+    return res
+
+
+def model_service_graph_delete(data, config_params):
+    if "graph_name" in data.keys():
+        graph_name = data["graph_name"]
+    else:
+        graph_name = "cyber"
+
+    db = config_params["db"]
+    res = db.delete_tables(graph_name)
+
+    return res
+
+
+def model_service_graph_update(data, config_params):
+    if "graph_name" in data.keys():
+        graph_name = data["graph_name"]
+    else:
+        graph_name = "cyber"
+
+    if "graph_cfg" in data.keys():
+        graph_cfg = data["graph_cfg"]
+    else:
+        graph_cfg = "cyber"
+
+    db = config_params["db"]
+    res = db.update_tables(graph_name, graph_cfg)
+
+    return res
 
 def model_service_graph_search_multi_hop_multi_edge(data, config_params):
     '''
@@ -41,8 +220,8 @@ def model_service_graph_search_multi_hop_multi_edge(data, config_params):
     # client = Client('10.217.62.41')
     # graph = CHGraph(client)
     graph = config_params["graph"]
-
-    graph.use_graph(graph_name)
+    db = config_params["db"]
+    graph.use_graph(graph_name, db)
 
     if "start_vertex_list" in data.keys():
         start_vertex_list = data["start_vertex_list"]
@@ -87,49 +266,57 @@ def model_service_graph_search_multi_hop_multi_edge(data, config_params):
     global res
     try:
         res = graph.multi_hop_multi_edge(step,
-                                 start_vertex_list,
-                                 direction,
-                                 edge_name_list,
-                                 edge_con_list_list,
-                                 target_field_list,
-                                 only_last_step,
-                                 plus_last_vertexes=plus_last_vertexes
-                                )
+                                         start_vertex_list,
+                                         direction,
+                                         edge_name_list,
+                                         edge_con_list_list,
+                                         target_field_list,
+                                         only_last_step,
+                                         plus_last_vertexes=plus_last_vertexes
+                                         )
     except Exception as e:
         print(e)
-   
 
-    #print(res[1])
+    # print(res[1])
     result = dict()
-    #result["result"] = res_list2
-    #res_list2 = [x.to_dict() for x in res_list2]
+    # result["result"] = res_list2
+    # res_list2 = [x.to_dict() for x in res_list2]
     if only_last_step:
         if plus_last_vertexes:
             edge = res[0]
             vertex = res[1]
             for i in range(len(edge_name_list)):
-                result["hop_"+str(step)] = {}
-                result["hop_"+str(step)][edge_name_list[i]] = edge[i].values.tolist()
-            result["last_step_vertex"] = vertex
+                result["hop_" + str(step)] = {}
+                result["hop_" + str(step)][edge_name_list[i]] = edge[i].values.tolist()
+            if type(vertex) is np.ndarray:
+                result["last_step_vertex"] = vertex.tolist()
+            else:
+                result["last_step_vertex"] = vertex
         else:
             edge = res
             for i in range(len(edge_name_list)):
-                result["hop_"+str(step)] = {}
-                result["hop_"+str(step)][edge_name_list[i]] = edge[i].values.tolist()
+                result["hop_" + str(step)] = {}
+                result["hop_" + str(step)][edge_name_list[i]] = edge[i].values.tolist()
     else:
-        if len(res) == 0 or res == [[[] for i in range(step)]]:
+        is_empty = True
+        is_blank_list = True
+        for i in range(len(res)):
+            for j in range(len(res[i])):
+                is_empty = is_empty & res[i][j].empty
+        # if len(res) == 0 or is_empty or res == [[[] for i in range(step)]]:
+        if len(res) == 0 or is_empty:
             for i in range(step):
-                result["hop_"+str(i+1)] = {}
+                result["hop_" + str(i + 1)] = {}
                 for j in range(len(edge_name_list)):
-                    result["hop_"+str(i+1)][edge_name_list[j]] = []
+                    result["hop_" + str(i + 1)][edge_name_list[j]] = []
         else:
             for i in range(step):
-                result["hop_"+str(i+1)] = {}
+                result["hop_" + str(i + 1)] = {}
                 for j in range(len(edge_name_list)):
-                    if len(res[i])==0:
-                        result["hop_"+str(i+1)][edge_name_list[j]] = []
+                    if len(res[i]) == 0:
+                        result["hop_" + str(i + 1)][edge_name_list[j]] = []
                     else:
-                        result["hop_"+str(i+1)][edge_name_list[j]] = res[i][j].values.tolist()
+                        result["hop_" + str(i + 1)][edge_name_list[j]] = res[i][j].values.tolist()
     return result
 
 
@@ -149,7 +336,7 @@ def model_service_graph_search_one_hop_multi_edge(data, config_params):
     }
     '''
 
-    #print("model_service_graph_search_one_hop_multi_edge")
+    # print("model_service_graph_search_one_hop_multi_edge")
 
     if "graph_name" in data.keys():
         graph_name = data["graph_name"]
@@ -161,11 +348,12 @@ def model_service_graph_search_one_hop_multi_edge(data, config_params):
 
     graph = config_params["graph"]
 
-    #graph_dir = "./config/tcpflow_flow.cfg.json"
+    # graph_dir = "./config/tcpflow_flow.cfg.json"
 
-    #graph = CHGraph(graph_dir, client)
+    # graph = CHGraph(graph_dir, client)
 
-    graph.use_graph(graph_name)
+    db = config_params["db"]
+    graph.use_graph(graph_name, db)
 
     if "start_vertex_list" in data.keys():
         start_vertex_list = data["start_vertex_list"]
@@ -197,7 +385,6 @@ def model_service_graph_search_one_hop_multi_edge(data, config_params):
     else:
         end_vertex_con_list = None
 
-
     # if "edge_con_list_tcpflow" in data.keys():
     #     edge_con_list_tcpflow = data["edge_con_list_tcpflow"]
     # else:
@@ -208,22 +395,21 @@ def model_service_graph_search_one_hop_multi_edge(data, config_params):
     # else:
     #     edge_con_list_flow = ["record_date='2019-04-15'"]
 
-    
     res_list2 = graph.one_hop_multi_edge(start_vertex_list,
                                          direction,
-                           edge_name_list,
-                           edge_con_list_list,
-                           target_field_list,
-                           end_vertex_con_list
-                           )
+                                         edge_name_list,
+                                         edge_con_list_list,
+                                         target_field_list,
+                                         end_vertex_con_list
+                                         )
 
     print("res_list2")
     print(res_list2)
 
     result = dict()
-    #result["result"] = res_list2
+    # result["result"] = res_list2
     for i in range(len(edge_name_list)):
-        result[edge_name_list[i]] = res_list2[i].values.tolist()    
+        result[edge_name_list[i]] = res_list2[i].values.tolist()
 
     return result
 
@@ -249,11 +435,12 @@ def model_service_graph_search_one_hop(data, config_params):
     # graph = CHGraph(client)
     graph = config_params["graph"]
 
-    #graph_dir = "./config/tcpflow_flow.cfg.json"
+    # graph_dir = "./config/tcpflow_flow.cfg.json"
 
-    #graph = CHGraph(graph_dir, client)
+    # graph = CHGraph(graph_dir, client)
 
-    graph.use_graph(graph_name)
+    db = config_params["db"]
+    graph.use_graph(graph_name, db)
 
     if "start_vertex_list" in data.keys():
         start_vertex_list = data["start_vertex_list"]
@@ -289,8 +476,8 @@ def model_service_graph_search_one_hop(data, config_params):
                               edge_name, edge_con_list, target_field_list,
                               end_vertex_con_list)
 
-    #result = dict()
-    #result["result"] = res_list2
+    # result = dict()
+    # result["result"] = res_list2
 
     return res_list2
 
@@ -326,11 +513,11 @@ def model_service_graph_search_multi_hop(data, config_params):
 
     graph = config_params["graph"]
 
-    #graph_dir = "./config/tcpflow_flow.cfg.json"
+    # graph_dir = "./config/tcpflow_flow.cfg.json"
 
-    #graph = CHGraph(graph_dir, client)
-
-    graph.use_graph(graph_name)
+    # graph = CHGraph(graph_dir, client)
+    db = config_params["db"]
+    graph.use_graph(graph_name, db)
 
     if "step" in data.keys():
         step = data["step"]
@@ -378,18 +565,18 @@ def model_service_graph_search_multi_hop(data, config_params):
         only_last_step = True
 
     res_list2 = graph.multi_hop(
-                  step,
-                  start_vertex_list,
-                  direction,
-                  edge_name,
-                  edge_con_list, 
-                  target_field_list,
-                  only_last_step,
-                  plus_last_vertexes=plus_last_vertexes,
-                  end_vertex_con_list=end_vertex_con_list)
+        step,
+        start_vertex_list,
+        direction,
+        edge_name,
+        edge_con_list,
+        target_field_list,
+        only_last_step,
+        plus_last_vertexes=plus_last_vertexes,
+        end_vertex_con_list=end_vertex_con_list)
 
-    #result = dict()
-    #result["result"] = res_list2
+    # result = dict()
+    # result["result"] = res_list2
 
     return res_list2
 
@@ -422,7 +609,8 @@ def model_service_graph_search_multi_hop_common_vertexes(data, config_params):
 
     graph = config_params["graph"]
 
-    graph.use_graph(graph_name)
+    db = config_params["db"]
+    graph.use_graph(graph_name, db)
 
     if "step" in data.keys():
         step = data["step"]
@@ -455,8 +643,8 @@ def model_service_graph_search_multi_hop_common_vertexes(data, config_params):
                                                 edge_name,
                                                 edge_con_list)
 
-    #result = dict()
-    #result["result"] = res_list2
+    # result = dict()
+    # result["result"] = res_list2
 
     return res_list2
 
@@ -484,11 +672,12 @@ def model_service_graph_search_match_edge(data, config_params):
     # graph = CHGraph(client)
     graph = config_params["graph"]
 
-    #graph_dir = "./config/tcpflow_flow.cfg.json"
+    # graph_dir = "./config/tcpflow_flow.cfg.json"
 
-    #graph = CHGraph(graph_dir, client)
+    # graph = CHGraph(graph_dir, client)
 
-    graph.use_graph(graph_name)
+    db = config_params["db"]
+    graph.use_graph(graph_name, db)
 
     if "edge_name" in data.keys():
         edge_name = data["edge_name"]
@@ -498,7 +687,7 @@ def model_service_graph_search_match_edge(data, config_params):
     if "edge_con_list" in data.keys():
         edge_con_list = data["edge_con_list"]
     else:
-        edge_con_list = ["downlink_length>100000000"]
+        edge_con_list = None
 
     if "target_field_list" in data.keys():
         target_field_list = data["target_field_list"]
@@ -507,14 +696,13 @@ def model_service_graph_search_match_edge(data, config_params):
 
     res_list2 = graph.match_edge(edge_name, edge_con_list, target_field_list)
 
-    #result = dict()
-    #result["result"] = res_list2
+    # result = dict()
+    # result["result"] = res_list2
 
     return res_list2
 
 
 def model_service_graph_search_match_vertex(data, config_params):
-
     if "graph_name" in data.keys():
         graph_name = data["graph_name"]
     else:
@@ -525,11 +713,12 @@ def model_service_graph_search_match_vertex(data, config_params):
 
     graph = config_params["graph"]
 
-    #graph_dir = "./config/tcpflow_flow.cfg.json"
+    # graph_dir = "./config/tcpflow_flow.cfg.json"
 
-    #graph = CHGraph(graph_dir, client)
+    # graph = CHGraph(graph_dir, client)
 
-    graph.use_graph(graph_name)
+    db = config_params["db"]
+    graph.use_graph(graph_name, db)
 
     if "vertex_name" in data.keys():
         vertex_name = data["vertex_name"]
@@ -547,7 +736,6 @@ def model_service_graph_search_match_vertex(data, config_params):
         target_field_list = ['record_time']
 
     res_list2 = graph.match_vertex(vertex_name, vertex_con_list, target_field_list)
-    
 
     return res_list2
 
@@ -578,8 +766,8 @@ def model_service_graph_search_multi_hop_multi_edge_common_vertexes(data, config
     # client = Client('10.217.62.41')
     # graph = CHGraph(client)
     graph = config_params["graph"]
-
-    graph.use_graph(graph_name)
+    db = config_params["db"]
+    graph.use_graph(graph_name, db)
 
     if "start_vertex_list" in data.keys():
         start_vertex_list = data["start_vertex_list"]
@@ -606,19 +794,17 @@ def model_service_graph_search_multi_hop_multi_edge_common_vertexes(data, config
     else:
         edge_con_list_list = [["downlink_length>10000", "protocol='http'"], ["record_date='2019-04-15'"]]
 
-
     res_list2 = graph.multi_hop_multi_edge_common_vertexes(step,
-                                             start_vertex_list,
-                                             direction,
-                                             edge_name_list,
-                                             edge_con_list_list,
-                                             )
+                                                           start_vertex_list,
+                                                           direction,
+                                                           edge_name_list,
+                                                           edge_con_list_list,
+                                                           )
 
     return res_list2
 
 
 def model_service_graph_search_find_path(data, config_params):
-
     if "graph_name" in data.keys():
         graph_name = data["graph_name"]
     else:
@@ -627,8 +813,8 @@ def model_service_graph_search_find_path(data, config_params):
     # client = Client('10.217.62.41')
     # graph = CHGraph(client)
     graph = config_params["graph"]
-
-    graph.use_graph(graph_name)
+    db = config_params["db"]
+    graph.use_graph(graph_name, db)
 
     if "start_vertex" in data.keys():
         start_vertex = data["start_vertex"]
@@ -660,21 +846,20 @@ def model_service_graph_search_find_path(data, config_params):
     else:
         step_limit = 2
 
-
-    res_list2 = graph.find_path_multi_edge(start_vertex, end_vertex, edge_name_list, edge_con_list_list, target_field_list, step_limit)
+    res_list2 = graph.find_path_multi_edge(start_vertex, end_vertex, edge_name_list, edge_con_list_list,
+                                           target_field_list, step_limit)
 
     # print("res_list2")
     # print(res_list2)
 
-    #result = dict()
+    # result = dict()
 
-    #result["result"] = res_list2
+    # result["result"] = res_list2
 
     return res_list2
 
 
 def model_service_register_graph(data, config_params):
-
     if "graph_name" in data.keys():
         graph_name = data["graph_name"]
     else:
@@ -685,8 +870,6 @@ def model_service_register_graph(data, config_params):
         if not is_json(graph_cfg):
             return "schema is not json string"
 
-    # client = Client('10.217.62.41')
-    # graph = CHGraph(client)
     graph = config_params["graph"]
     res = graph.register_graph(graph_name, graph_cfg)
 
@@ -694,7 +877,6 @@ def model_service_register_graph(data, config_params):
 
 
 def model_service_delete_graph(data, config_params):
-
     if "graph_name" in data.keys():
         graph_name = data["graph_name"]
     else:
@@ -710,7 +892,6 @@ def model_service_delete_graph(data, config_params):
 
 
 def model_service_show_graph(config_params):
-
     # client = Client('10.217.62.41')
     #
     # graph = CHGraph(client)
@@ -720,6 +901,7 @@ def model_service_show_graph(config_params):
 
     return list(res["graph_name"])
 
+
 '''
 def model_service_summary_graph(data, config_params):
 
@@ -737,24 +919,20 @@ def model_service_summary_graph(data, config_params):
 
 
 def model_service_summary_graph(graph_name, config_params):
+    client = Client('10.217.62.41')
 
-    # client = Client('10.217.62.41')
-    #
-    # graph = CHGraph(client)
-
-    graph = config_params["graph"]
+    graph = CHGraph(client)
 
     res = graph.summary_graph(graph_name)
 
     if res != None:
-        return {"edge": res[0], "vertex":res[1]}
+        return {"edge": res[0], "vertex": res[1]}
     else:
-        return "graph name ["+graph_name+"] does not exist"
+        return "graph name [" + graph_name + "] does not exist"
 
 
 def model_service_description_graph(graph_name, config_params):
     # client = Client('10.217.62.41')
-    #
     # graph = CHGraph(client)
     graph = config_params["graph"]
 
@@ -764,16 +942,13 @@ def model_service_description_graph(graph_name, config_params):
 
 
 def model_service_insert_edge(data, config_params):
-
     if "graph_name" in data.keys():
         graph_name = data["graph_name"]
     else:
         graph_name = "cyber_plus"
 
     # client = Client('10.217.62.41')
-    #
     # graph = CHGraph(client)
-
     graph = config_params["graph"]
 
     if "edge_name" in data.keys():
@@ -784,7 +959,8 @@ def model_service_insert_edge(data, config_params):
     if "edge_schema" in data.keys():
         edge_schema = data["edge_schema"]
     else:
-        edge_schema = ["record_time", "record_date", "source_ip", "destination_ip", "protocol", "destination_port", "uplink_length", "downlink_length"]
+        edge_schema = ["record_time", "record_date", "source_ip", "destination_ip", "protocol", "destination_port",
+                       "uplink_length", "downlink_length"]
 
     if "edge_data" in data.keys():
         edge_data = data["edge_data"]
@@ -793,21 +969,20 @@ def model_service_insert_edge(data, config_params):
     else:
         edge_data = [("2019-04-11 18:48:59", "2019-04-11", "10.66.18.32", "184.173.90.200", "http", "80", 14725, 3116)]
 
-    graph.use_graph(graph_name)
+    db = config_params["db"]
+    graph.use_graph(graph_name, db)
     res = graph.insert_edge(edge_name, edge_schema, edge_data)
 
     return res
 
 
 def model_service_insert_vertex(data, config_params):
-
     if "graph_name" in data.keys():
         graph_name = data["graph_name"]
     else:
         graph_name = "cyber_plus"
 
     # client = Client('10.217.62.41')
-    #
     # graph = CHGraph(client)
     graph = config_params["graph"]
 
@@ -826,16 +1001,17 @@ def model_service_insert_vertex(data, config_params):
         for i, item_vertex in enumerate(vertex_data):
             vertex_data[i] = tuple(item_vertex)
     else:
-        vertex_data = [("2021-01 04","1.1.1.1","p47708v.hulk.shbt.qihoo.net","2"),("2021-01-05","1.1.1.2","p47709v.hulk.shbt.qihoo.net","3")]
+        vertex_data = [("2021-01 04", "1.1.1.1", "p47708v.hulk.shbt.qihoo.net", "2"),
+                       ("2021-01-05", "1.1.1.2", "p47709v.hulk.shbt.qihoo.net", "3")]
 
-    graph.use_graph(graph_name)
+    db = config_params["db"]
+    graph.use_graph(graph_name, db)
     res = graph.insert_vertex(vertex_name, vertex_schema, vertex_data)
 
     return res
 
 
 def model_service_path_finding(data, config_params):
-
     if "graph_name" in data.keys():
         graph_name = data["graph_name"]
     else:
@@ -876,50 +1052,52 @@ def model_service_path_finding(data, config_params):
     else:
         step_limit = 1
 
-    graph.use_graph(graph_name)
+    db = config_params["db"]
+    graph.use_graph(graph_name, db)
 
-    res = graph.find_path_multi_edge(start_vertex_list, end_vertex_list, edge_name_list, edge_con_list_list, target_field_list, step_limit)
+    res = graph.find_path_multi_edge(start_vertex_list, end_vertex_list, edge_name_list, edge_con_list_list,
+                                     target_field_list, step_limit)
 
     print("res3")
     print(res)
 
-    #try:
+    # try:
     #    res = graph.find_path_multi_edge(start_vertex_list, end_vertex_list, edge_name_list, edge_con_list_list, target_field_list, step_limit)
-    #except Exception as e:
+    # except Exception as e:
     #    print(e)
     #    return "Find path multi edge failed."
 
-
     result = {}
 
     if step_limit == 1:
-        result["hop_"+str(1)] = {}
+        result["hop_" + str(1)] = {}
         for j in range(len(edge_name_list)):
-            result["hop_"+str(1)][edge_name_list[j]] = res[0][j].values.tolist()
+            result["hop_" + str(1)][edge_name_list[j]] = res[0][j].values.tolist()
     elif step_limit >= 2:
-        for i in range(1, step_limit+1):
-            result["s_"+str(i)] = {}
+        for i in range(1, step_limit + 1):
+            result["s_" + str(i)] = {}
             if i == 1:
-                result["s_"+str(i)] = {}
+                result["s_" + str(i)] = {}
                 for j in range(len(edge_name_list)):
-                    result["s_"+str(i)][edge_name_list[j]] = res[i-1][j].values.tolist()
+                    result["s_" + str(i)][edge_name_list[j]] = res[i - 1][j].values.tolist()
             else:
-                result["s_"+str(i)] = {}
+                result["s_" + str(i)] = {}
                 for h in range(step_limit):
-                    result["s_" + str(i)]["hop_"+str(h+1)] = {}
+                    result["s_" + str(i)]["hop_" + str(h + 1)] = {}
                     for j in range(len(edge_name_list)):
-                        if h < len(res[i-1]):
-                            result["s_"+str(i)]["hop_"+str(h+1)][edge_name_list[j]] = res[i-1][h][j].values.tolist()
-                            result["s_"+str(i)]["hop_"+str(h+1)][edge_name_list[j]] = res[i-1][h][j].values.tolist()
+                        if h < len(res[i - 1]):
+                            result["s_" + str(i)]["hop_" + str(h + 1)][edge_name_list[j]] = res[i - 1][h][
+                                j].values.tolist()
+                            result["s_" + str(i)]["hop_" + str(h + 1)][edge_name_list[j]] = res[i - 1][h][
+                                j].values.tolist()
                         else:
-                            result["s_"+str(i)]["hop_"+str(h+1)][edge_name_list[j]] = []
-                            result["s_"+str(i)]["hop_"+str(h+1)][edge_name_list[j]] = []
+                            result["s_" + str(i)]["hop_" + str(h + 1)][edge_name_list[j]] = []
+                            result["s_" + str(i)]["hop_" + str(h + 1)][edge_name_list[j]] = []
 
     return result
 
 
 def model_service_create_subgraph(data, config_params):
-
     if "graph_name" in data.keys():
         graph_name = data["graph_name"]
     else:
@@ -935,7 +1113,8 @@ def model_service_create_subgraph(data, config_params):
     # graph = CHGraph(client)
     graph = config_params["graph"]
 
-    graph.use_graph(graph_name)
+    db = config_params["db"]
+    graph.use_graph(graph_name, db)
 
     try:
         status = graph.create_subgraph(subgraph_name)
@@ -945,7 +1124,6 @@ def model_service_create_subgraph(data, config_params):
 
 
 def model_service_update_subgraph_by_multi_hop_multi_edge(data, config_params):
-
     if "graph_name" in data.keys():
         graph_name = data["graph_name"]
     else:
@@ -956,7 +1134,8 @@ def model_service_update_subgraph_by_multi_hop_multi_edge(data, config_params):
     # graph = CHGraph(client)
     graph = config_params["graph"]
 
-    graph.use_graph(graph_name)
+    db = config_params["db"]
+    graph.use_graph(graph_name, db)
 
     if "subgraph_name" in data.keys():
         subgraph_name = data["subgraph_name"]
@@ -988,13 +1167,10 @@ def model_service_update_subgraph_by_multi_hop_multi_edge(data, config_params):
     else:
         edge_con_list_list = [["downlink_length>10000", "protocol='http'"], ["record_date='2019-04-15'"]]
 
-    client = Client('10.217.62.41')
+    # client = Client('10.217.62.41')
+    # graph = CHGraph(client)
 
-    graph = CHGraph(client)
-
-    graph.use_graph(graph_name)
-
-    #status = graph.update_subgraph_by_multi_hop_multi_edge(
+    # status = graph.update_subgraph_by_multi_hop_multi_edge(
     #       subgraph_name,
     #       step,
     #       start_vertex_list,
@@ -1004,22 +1180,21 @@ def model_service_update_subgraph_by_multi_hop_multi_edge(data, config_params):
     #   )
 
     try:
-       status = graph.update_subgraph_by_multi_hop_multi_edge(
-           subgraph_name,
-           step,
-           start_vertex_list,
-           direction,
-           edge_name_list,
-           edge_con_list_list
-       )
-       return status
+        status = graph.update_subgraph_by_multi_hop_multi_edge(
+            subgraph_name,
+            step,
+            start_vertex_list,
+            direction,
+            edge_name_list,
+            edge_con_list_list
+        )
+        return status
     except Exception as e:
-       print(e)
-       return "update subgraph by multi hop multi edge failed"
+        print(e)
+        return "update subgraph by multi hop multi edge failed"
 
 
 def model_service_update_subgraph_by_match_edge(data, config_params):
-
     if "graph_name" in data.keys():
         graph_name = data["graph_name"]
     else:
@@ -1047,8 +1222,8 @@ def model_service_update_subgraph_by_match_edge(data, config_params):
     #
     # graph = CHGraph(client)
     graph = config_params["graph"]
-
-    graph.use_graph(graph_name)
+    db = config_params["db"]
+    graph.use_graph(graph_name, db)
 
     try:
         status = graph.update_subgraph_by_match_edge(subgraph_name, edge_name, edge_con_list)
@@ -1059,7 +1234,6 @@ def model_service_update_subgraph_by_match_edge(data, config_params):
 
 
 def model_service_update_subgraph_by_find_path_multi_edge(data, config_params):
-
     if "graph_name" in data.keys():
         graph_name = data["graph_name"]
     else:
@@ -1074,8 +1248,8 @@ def model_service_update_subgraph_by_find_path_multi_edge(data, config_params):
     #
     # graph = CHGraph(client)
     graph = config_params["graph"]
-
-    graph.use_graph(graph_name)
+    db = config_params["db"]
+    graph.use_graph(graph_name, db)
 
     if "start_vertex_list" in data.keys():
         start_vertex_list = data["start_vertex_list"]
@@ -1095,29 +1269,28 @@ def model_service_update_subgraph_by_find_path_multi_edge(data, config_params):
     if "edge_con_list_list" in data.keys():
         edge_con_list_list = data["edge_con_list_list"]
     else:
-        edge_con_list_list = [["record_date='2017-5-13'"],["record_date='2017-5-9'"]]
+        edge_con_list_list = [["record_date='2017-5-13'"], ["record_date='2017-5-9'"]]
 
     if "step_limit" in data.keys():
         step_limit = data["step_limit"]
     else:
         step_limit = 1
 
-
     try:
         graph.update_subgraph_by_find_path_multi_edge(subgraph_name,
-                                                start_vertex_list,
-                                                end_vertex_list,
-                                                edge_name_list,
-                                                edge_con_list_list,
-                                                step_limit)
+                                                      start_vertex_list,
+                                                      end_vertex_list,
+                                                      edge_name_list,
+                                                      edge_con_list_list,
+                                                      step_limit)
         return "subgraph by find path multi edge success"
     except Exception as e:
         print(e)
         return "subgraph by find path multi edge failed"
     return
 
-def model_service_destroy_subgraph(data, config_params):
 
+def model_service_destroy_subgraph(data, config_params):
     if "graph_name" in data.keys():
         graph_name = data["graph_name"]
     else:
@@ -1132,8 +1305,8 @@ def model_service_destroy_subgraph(data, config_params):
     #
     # graph = CHGraph(client)
     graph = config_params["graph"]
-
-    graph.use_graph(graph_name)
+    db = config_params["db"]
+    graph.use_graph(graph_name, db)
 
     try:
         status = graph.destroy_subgraph(subgraph_name)
@@ -1143,7 +1316,6 @@ def model_service_destroy_subgraph(data, config_params):
 
 
 def model_service_metric_indegree(data, config_params):
-
     if "graph_name" in data.keys():
         graph_name = data["graph_name"]
     else:
@@ -1168,8 +1340,8 @@ def model_service_metric_indegree(data, config_params):
     #
     # graph = CHGraph(client)
     graph = config_params["graph"]
-
-    graph.use_graph(graph_name)
+    db = config_params["db"]
+    graph.use_graph(graph_name, db)
 
     try:
         status = graph.metric_indegree(edge_name, if_sort, topk)
@@ -1179,7 +1351,6 @@ def model_service_metric_indegree(data, config_params):
 
 
 def model_service_metric_outdegree(data, config_params):
-
     if "graph_name" in data.keys():
         graph_name = data["graph_name"]
     else:
@@ -1204,8 +1375,8 @@ def model_service_metric_outdegree(data, config_params):
     #
     # graph = CHGraph(client)
     graph = config_params["graph"]
-
-    graph.use_graph(graph_name)
+    db = config_params["db"]
+    graph.use_graph(graph_name, db)
 
     try:
         status = graph.metric_outdegree(edge_name, if_sort, topk)
@@ -1215,7 +1386,6 @@ def model_service_metric_outdegree(data, config_params):
 
 
 def model_service_metric_degree(data, config_params):
-
     if "graph_name" in data.keys():
         graph_name = data["graph_name"]
     else:
@@ -1240,8 +1410,8 @@ def model_service_metric_degree(data, config_params):
     #
     # graph = CHGraph(client)
     graph = config_params["graph"]
-
-    graph.use_graph(graph_name)
+    db = config_params["db"]
+    graph.use_graph(graph_name, db)
 
     try:
         status = graph.metric_degree(edge_name, if_sort, topk)
@@ -1285,8 +1455,8 @@ def model_service_metric_pagerank(data, config_params):
     #
     # graph = CHGraph(client)
     graph = config_params["graph"]
-
-    graph.use_graph(graph_name)
+    db = config_params["db"]
+    graph.use_graph(graph_name, db)
 
     try:
         status = graph.metric_pagerank(edge_name, d, num_iter, if_sort, topk)
@@ -1300,41 +1470,51 @@ def model_service_vertex_match_property(data, config_params):
     if "graph_name" in data.keys():
         graph_name = data["graph_name"]
     else:
-        graph_name = "anti_money_launder"
+        return "graph_name shouldn't be null"
 
     if "vertex_id_list" in data.keys():
         vertex_id_list = data["vertex_id_list"]
     else:
-        vertex_id_list = ['119309','151550']
+        return "vertex_id_list shouldn't be null"
 
     if "vertex_name" in data.keys():
         vertex_name = data["vertex_name"]
     else:
-        vertex_name = "account"
+        return "vertex_name shouldn't be null"
 
     if "vertex_con_list" in data.keys():
         vertex_con_list = data["vertex_con_list"]
     else:
-        vertex_con_list = ["record_date < \'2017-05-11\'"]
+        vertex_con_list = None
 
     if "target_field_list" in data.keys():
         target_field_list = data["target_field_list"]
     else:
         target_field_list = None
+    if "page" in data.keys():
+        page = data["page"]
+    else:
+        page = None
+    if "page_size" in data.keys():
+        page_size = data["page_size"]
+    else:
+        page_size = None
 
     # client = Client('10.217.62.41')
     #
     # graph = CHGraph(client)
     graph = config_params["graph"]
-
-    graph.use_graph(graph_name)
+    db = config_params["db"]
+    graph.use_graph(graph_name, db)
 
     try:
         vertex_property = graph.vertex_match_property(vertex_id_list,
-                                             vertex_name,
-                                             vertex_con_list,
-                                             target_field_list
-                                             )
+                                                      vertex_name,
+                                                      vertex_con_list,
+                                                      target_field_list,
+                                                      page,
+                                                      page_size,
+                                                      )
         return vertex_property
     except Exception as e:
         print(e)
@@ -1345,22 +1525,22 @@ def model_service_edge_match_property(data, config_params):
     if "graph_name" in data.keys():
         graph_name = data["graph_name"]
     else:
-        graph_name = "anti_money_launder"
+        return "graph_name shouldn't be null"
 
     if "start_vertex_list" in data.keys():
         start_vertex_list = data["start_vertex_list"]
     else:
-        start_vertex_list = ['770305']
+        return "start_vertex_list shouldn't be null"
 
     if "end_vertex_list" in data.keys():
         end_vertex_list = data["end_vertex_list"]
     else:
-        end_vertex_list = ["402592"]
+        return "end_vertex_list shouldn't be null"
 
     if "edge_name" in data.keys():
         edge_name = data["edge_name"]
     else:
-        edge_name = "transactions"
+        return "edge_name shouldn't be null"
 
     if "edge_con_list" in data.keys():
         edge_con_list = data["edge_con_list"]
@@ -1371,13 +1551,20 @@ def model_service_edge_match_property(data, config_params):
         target_field_list = data["target_field_list"]
     else:
         target_field_list = None
-
+    if "page" in data.keys():
+        page = data["page"]
+    else:
+        page = None
+    if "page_size" in data.keys():
+        page_size = data["page_size"]
+    else:
+        page_size = None
     # client = Client('10.217.62.41')
     #
     # graph = CHGraph(client)
     graph = config_params["graph"]
-
-    graph.use_graph(graph_name)
+    db = config_params["db"]
+    graph.use_graph(graph_name, db)
 
     try:
         edge_property = graph.edge_match_property(
@@ -1385,7 +1572,9 @@ def model_service_edge_match_property(data, config_params):
             end_vertex_list,
             edge_name,
             edge_con_list,
-            target_field_list
+            target_field_list,
+            page,
+            page_size
         )
         return edge_property
     except Exception as e:
@@ -1407,7 +1596,7 @@ def model_service_query_vertexes(data, config_params):
     if "vertex_con_list" in data.keys():
         vertex_con_list = data["vertex_con_list"]
     else:
-        vertex_con_list = ["record_date < \'2017-05-11\'"]
+        vertex_con_list = None
 
     if "target_field_list" in data.keys():
         target_field_list = data["target_field_list"]
@@ -1418,8 +1607,8 @@ def model_service_query_vertexes(data, config_params):
     #
     # graph = CHGraph(client)
     graph = config_params["graph"]
-
-    graph.use_graph(graph_name)
+    db = config_params["db"]
+    graph.use_graph(graph_name, db)
 
     try:
         vertex_query = graph.query_vertexes(
@@ -1458,8 +1647,8 @@ def model_service_query_edges(data, config_params):
     #
     # graph = CHGraph(client)
     graph = config_params["graph"]
-
-    graph.use_graph(graph_name)
+    db = config_params["db"]
+    graph.use_graph(graph_name, db)
 
     try:
         edges_result = graph.query_edges(
@@ -1503,8 +1692,8 @@ def model_service_time_static_subgraph(data, config_params):
     #
     # graph = CHGraph(client)
     graph = config_params["graph"]
-
-    graph.use_graph(graph_name)
+    db = config_params["db"]
+    graph.use_graph(graph_name, db)
 
     try:
         time_static_result = graph.count_edge_by_time(
